#|
  Step-gradients/theme.jl

  Copyright (C) 1999 Michele Campeotto <micampe@interpuntonet.it>

  This theme is based on WindowMaker, the GNUStep project window manager.
  I ported it to Sawmill because I liked his look'n'feel but Window Maker
  is'nt working well with Gnome.
  
  This is an improved version of my Step theme with gradients added.
  
  Credits go to (in no particular order):
   - John Harper         <john@dcs.warwick.ac.uk> (twm and Sawfish, of course)
   - Isaac Feitler       <ike@mit.edu>            (blueberry)
   - Pedro Lopes         <paol@teleweb.pt>        (Operational)

   - All the Window Maker team          http://www.windowmaker.org
|#

(require 'gradient)

;; --- Frame preferences ---------------------------------------------------

(defgroup Step-gradients-frame "Step-gradients"
  :group appearance)

(defcustom Step-gradients:title-alignement 'center
  "Alignement of the text in the titlebar"
  :group (appearance Step-gradients-frame)
  :type symbol
  :options (left center right)
  :after-set after-setting-frame-option)

(defcustom Step-gradients:title-active-color (get-color "#ffffffffffff")
  "Color of the active window title"
  :group (appearance Step-gradients-frame)
  :type color
  :after-set after-setting-frame-option)

(defcustom Step-gradients:title-inactive-color (get-color "#999999999999")
  "Color of inactive windows titles"
  :group (appearance Step-gradients-frame)
  :type color
  :after-set after-setting-frame-option)

(defcustom Step-gradients:flat-buttons t
  "Use flat buttons on titlebar"
  :group (appearance Step-gradients-frame)
  :type boolean
  :after-set after-setting-frame-option)

(defgroup Step-gradients-titlebar "Step-gradients titlebar"
  :group appearance)

(defcustom Step-gradients:titlebar-gradient-type 'horizontal
  "Gradient direction for the titlebar"
  :group (appearance Step-gradients-titlebar)
  :type symbol
  :options (none horizontal vertical)
  :after-set after-setting-frame-option)

(defcustom Step-gradients:titlebar-active-color (get-color "#ee1399990000")
  "'From' color of the active window titlebar"
  :group (appearance Step-gradients-titlebar)
  :type color
  :after-set after-setting-frame-option)

(defcustom Step-gradients:titlebar-active-color2 (get-color "#8ccc59990000")
  "'To' color of the active window titlebar"
  :group (appearance Step-gradients-titlebar)
  :type color
  :after-set after-setting-frame-option)

(defcustom Step-gradients:titlebar-inactive-color (get-color "#170a38515999")
  "'From' color of inactive windows titlebars"
  :group (appearance Step-gradients-titlebar)
  :type color
  :after-set after-setting-frame-option)

(defcustom Step-gradients:titlebar-inactive-color2 (get-color "#000000000000")
  "'To' color of inactive windows titlebars"
  :group (appearance Step-gradients-titlebar)
  :type color
  :after-set after-setting-frame-option)

(defgroup Step-gradients-resizebar "Step-gradients resizebar"
  :group appearance)

(defcustom Step-gradients:resizebar-gradient-type 'horizontal
  "Gradient direction for the resizebar"
  :group (appearance Step-gradients-resizebar)
  :type symbol
  :options (none horizontal vertical)
  :after-set after-setting-frame-option)

(defcustom Step-gradients:resizebar-active-color (get-color "#170a38515999")
  "'From' color of the active window resizebar"
  :group (appearance Step-gradients-resizebar)
  :type color
  :after-set after-setting-frame-option)

(defcustom Step-gradients:resizebar-active-color2 (get-color "#000000000000")
  "'To' color of the active window resizebar"
  :group (appearance Step-gradients-resizebar)
  :type color
  :after-set after-setting-frame-option)

(defcustom Step-gradients:resizebar-inactive-color (get-color "#170a38515999")
  "'From' color of inactive windows resizebars"
  :group (appearance Step-gradients-resizebar)
  :type color
  :after-set after-setting-frame-option)

(defcustom Step-gradients:resizebar-inactive-color2 (get-color "#000000000000")
  "'To' color of inactive windows resizebars"
  :group (appearance Step-gradients-resizebar)
  :type color
  :after-set after-setting-frame-option)

;; --- Frame definition ----------------------------------------------------

(let* (
    (title-font (get-font "-*-helvetica-bold-r-normal-*-12-*-*-*-*-*-*"))
    (title-colors (lambda ()
        (list Step-gradients:title-inactive-color
              Step-gradients:title-active-color)))
    (text-align (lambda () Step-gradients:title-alignement))
    (border-color (get-color "black"))

    (brighten-color 
        (lambda (color delta)
            (get-color-rgb
                (min 65535 (+ (car (color-rgb color)) delta))
                (min 65535 (+ (cadr (color-rgb color)) delta))
                (min 65535 (+ (car (cddr (color-rgb color))) delta))
            )))

    (darken-color 
        (lambda (color delta)
            (get-color-rgb
                (max 0 (- (car (color-rgb color)) delta))
                (max 0 (- (cadr (color-rgb color)) delta))
                (max 0 (- (car (cddr (color-rgb color))) delta))
            )))

    (render-titlebar
        (lambda (img state)
            (apply (cond
                ((eq Step-gradients:titlebar-gradient-type 'none)
                    draw-horizontal-gradient)
                ((eq Step-gradients:titlebar-gradient-type 'horizontal)
                    draw-horizontal-gradient)
                ((eq Step-gradients:titlebar-gradient-type 'vertical)
                    draw-vertical-gradient)
                ((eq Step-gradients:titlebar-gradient-type 'diagonal)
                    draw-diagonal-gradient))
            img
            (if state
                (if (eq Step-gradients:titlebar-gradient-type 'none)
                    (list Step-gradients:titlebar-active-color Step-gradients:titlebar-active-color)
                    (list Step-gradients:titlebar-active-color Step-gradients:titlebar-active-color2))
                (if (eq Step-gradients:titlebar-gradient-type 'none)
                    (list Step-gradients:titlebar-inactive-color Step-gradients:titlebar-inactive-color)
                    (list Step-gradients:titlebar-inactive-color Step-gradients:titlebar-inactive-color2))
            ))))

    (render-titlebar-bevel-up (lambda (img state)
        (apply draw-horizontal-gradient img
            (if state
                (cond
                    ((eq Step-gradients:titlebar-gradient-type 'none) (list 
                        (brighten-color
                            Step-gradients:titlebar-active-color 22000)
                        (brighten-color
                            Step-gradients:titlebar-active-color 22000)))
                    ((eq Step-gradients:titlebar-gradient-type 'vertical) (list 
                        (brighten-color
                            Step-gradients:titlebar-active-color 22000)
                        (brighten-color
                            Step-gradients:titlebar-active-color 22000)))
                    (t (list
                        (brighten-color
                            Step-gradients:titlebar-active-color 22000)
                        (brighten-color
                            Step-gradients:titlebar-active-color2 22000))))
                (cond
                    ((eq Step-gradients:titlebar-gradient-type 'none) (list
                        (brighten-color
                            Step-gradients:titlebar-inactive-color 22000)
                        (brighten-color
                            Step-gradients:titlebar-inactive-color 22000)))
                    ((eq Step-gradients:titlebar-gradient-type 'vertical) (list 
                        (brighten-color
                            Step-gradients:titlebar-inactive-color 22000)
                        (brighten-color
                            Step-gradients:titlebar-inactive-color 22000)))
                    (t (list
                        (brighten-color
                            Step-gradients:titlebar-inactive-color 22000)
                        (brighten-color
                            Step-gradients:titlebar-inactive-color2 22000))))))))

    (render-titlebar-bevel-down (lambda (img state)
        (apply draw-horizontal-gradient img
        (if state
            (cond
                ((eq Step-gradients:titlebar-gradient-type 'none) (list
                    (darken-color Step-gradients:titlebar-active-color 22000)
                    (darken-color Step-gradients:titlebar-active-color 22000)))
                ((eq Step-gradients:titlebar-gradient-type 'vertical) (list 
                    (darken-color Step-gradients:titlebar-active-color2 22000)
                    (darken-color Step-gradients:titlebar-active-color2 22000)))
                (t (list
                    (darken-color Step-gradients:titlebar-active-color 22000)
                    (darken-color Step-gradients:titlebar-active-color2 22000))))
            (cond
                ((eq Step-gradients:titlebar-gradient-type 'none) (list
                    (darken-color Step-gradients:titlebar-inactive-color 22000)
                    (darken-color Step-gradients:titlebar-inactive-color 22000)))
                ((eq Step-gradients:titlebar-gradient-type 'vertical) (list 
                    (darken-color Step-gradients:titlebar-inactive-color2 22000)
                    (darken-color Step-gradients:titlebar-inactive-color2 22000)))
                (t (list
                    (darken-color Step-gradients:titlebar-inactive-color 22000)
                    (darken-color Step-gradients:titlebar-inactive-color2 22000))))))))

    (render-resizebar (lambda (img state)
        (apply
            (cond
                ((eq Step-gradients:resizebar-gradient-type 'none)
                    draw-horizontal-gradient)
                ((eq Step-gradients:resizebar-gradient-type 'horizontal)
                    draw-horizontal-gradient)
                ((eq Step-gradients:resizebar-gradient-type 'vertical)
                    draw-vertical-gradient)
                ((eq Step-gradients:resizebar-gradient-type 'diagonal)
                    draw-diagonal-gradient)
            )
        img
        (if state
            (if (eq Step-gradients:resizebar-gradient-type 'none)
                (list Step-gradients:resizebar-active-color
                      Step-gradients:resizebar-active-color)
                (list Step-gradients:resizebar-active-color
                      Step-gradients:resizebar-active-color2))
            (if (eq Step-gradients:resizebar-gradient-type 'none)
                (list Step-gradients:resizebar-inactive-color
                      Step-gradients:resizebar-inactive-color)
                (list Step-gradients:resizebar-inactive-color
                      Step-gradients:resizebar-inactive-color2))))))

    (render-resizebar-bevel-up (lambda (img state)
        (apply draw-horizontal-gradient img
            (if state
                (cond
                    ((eq Step-gradients:resizebar-gradient-type 'none) (list
                        (brighten-color Step-gradients:resizebar-active-color 22000)
                        (brighten-color Step-gradients:resizebar-active-color 22000)))
                    ((eq Step-gradients:resizebar-gradient-type 'vertical) (list 
                        (brighten-color Step-gradients:resizebar-active-color 22000)
                        (brighten-color Step-gradients:resizebar-active-color 22000)))
                    (t (list
                        (brighten-color Step-gradients:resizebar-active-color 22000)
                        (brighten-color Step-gradients:resizebar-active-color2 22000))))
                (cond
                    ((eq Step-gradients:resizebar-gradient-type 'none) (list
                        (brighten-color Step-gradients:resizebar-inactive-color 22000)
                        (brighten-color Step-gradients:resizebar-inactive-color 22000)))
                    ((eq Step-gradients:resizebar-gradient-type 'vertical) (list 
                        (brighten-color Step-gradients:resizebar-inactive-color 22000)
                        (brighten-color Step-gradients:resizebar-inactive-color 22000)))
                    (t (list
                        (brighten-color Step-gradients:resizebar-inactive-color 22000)
                        (brighten-color Step-gradients:resizebar-inactive-color2 22000))))))))
                
    (render-resizebar-bevel-down (lambda (img state)
        (apply draw-horizontal-gradient img
            (if state
                (cond
                    ((eq Step-gradients:resizebar-gradient-type 'none) (list 
                        (darken-color Step-gradients:resizebar-active-color 22000)
                        (darken-color Step-gradients:resizebar-active-color 22000)))
                    ((eq Step-gradients:resizebar-gradient-type 'vertical) (list 
                        (darken-color Step-gradients:resizebar-active-color 22000)
                        (darken-color Step-gradients:resizebar-active-color 22000)))
                    (t (list
                        (darken-color Step-gradients:resizebar-active-color 22000)
                        (darken-color Step-gradients:resizebar-active-color2 22000))))
                    (cond
                    ((eq Step-gradients:resizebar-gradient-type 'none) (list
                        (darken-color Step-gradients:resizebar-inactive-color 22000)
                        (darken-color Step-gradients:resizebar-inactive-color 22000)))
                    ((eq Step-gradients:resizebar-gradient-type 'vertical) (list 
                        (darken-color Step-gradients:resizebar-inactive-color 22000)
                        (darken-color Step-gradients:resizebar-inactive-color 22000)))
                    (t (list
                        (darken-color Step-gradients:resizebar-inactive-color 22000)
                        (darken-color Step-gradients:resizebar-inactive-color2 22000))))))))

    (render-bottom-left (lambda (img state)
        (apply (cond
            ((eq Step-gradients:resizebar-gradient-type 'vertical)
                draw-vertical-gradient)
            (t
                draw-horizontal-gradient))
        img
        (if state
            (cond
                ((eq Step-gradients:resizebar-gradient-type 'vertical) (list
                    Step-gradients:resizebar-active-color
                    Step-gradients:resizebar-active-color2))
                (t (list
                    Step-gradients:resizebar-active-color
                    Step-gradients:resizebar-active-color)))
            (cond
                ((eq Step-gradients:resizebar-gradient-type 'vertical) (list
                    Step-gradients:resizebar-inactive-color
                    Step-gradients:resizebar-inactive-color2))
                (t (list
                    Step-gradients:resizebar-inactive-color
                    Step-gradients:resizebar-inactive-color)))))))

    (render-bottom-left-bevel-up (lambda (img state)
        (apply draw-vertical-gradient img
            (if state
                (cond
                    ((eq Step-gradients:resizebar-gradient-type 'vertical) (list 
                        (brighten-color Step-gradients:resizebar-active-color 22000)
                        (brighten-color Step-gradients:resizebar-active-color2 22000)))
                    (t (list
                        (brighten-color Step-gradients:resizebar-active-color 22000)
                        (brighten-color Step-gradients:resizebar-active-color 22000))))
                (cond
                    ((eq Step-gradients:resizebar-gradient-type 'vertical) (list 
                        (brighten-color Step-gradients:resizebar-inactive-color 22000)
                        (brighten-color Step-gradients:resizebar-inactive-color2 22000)))
                    (t (list
                        (brighten-color Step-gradients:resizebar-inactive-color 22000)
                        (brighten-color Step-gradients:resizebar-inactive-color 22000))))))))

    (render-bottom-left-bevel-down (lambda (img state)
        (apply draw-vertical-gradient img
            (if state
                (cond
                    ((eq Step-gradients:resizebar-gradient-type 'vertical) (list 
                        (darken-color Step-gradients:resizebar-active-color 22000)
                        (darken-color Step-gradients:resizebar-active-color2 22000)))
                    (t (list
                        (darken-color Step-gradients:resizebar-active-color 22000)
                        (darken-color Step-gradients:resizebar-active-color 22000))))
                (cond
                    ((eq Step-gradients:resizebar-gradient-type 'vertical) (list 
                        (darken-color Step-gradients:resizebar-inactive-color 22000)
                        (darken-color Step-gradients:resizebar-inactive-color2 22000)))
                    (t (list
                        (darken-color Step-gradients:resizebar-inactive-color 22000)
                        (darken-color Step-gradients:resizebar-inactive-color 22000))))))))

    (render-bottom-right (lambda (img state)
        (apply
            (cond
                ((eq Step-gradients:resizebar-gradient-type 'vertical)
                    draw-vertical-gradient)
                (t
                    draw-horizontal-gradient))
            img
            (if state
                (cond
                    ((eq Step-gradients:resizebar-gradient-type 'vertical) (list
                        Step-gradients:resizebar-active-color
                        Step-gradients:resizebar-active-color2))
                    ((eq Step-gradients:resizebar-gradient-type 'none) (list
                        Step-gradients:resizebar-active-color
                        Step-gradients:resizebar-active-color))
                    (t (list
                        Step-gradients:resizebar-active-color2
                        Step-gradients:resizebar-active-color2)))
                (cond
                    ((eq Step-gradients:resizebar-gradient-type 'vertical) (list
                        Step-gradients:resizebar-inactive-color
                        Step-gradients:resizebar-inactive-color2))
                    ((eq Step-gradients:resizebar-gradient-type 'none) (list
                        Step-gradients:resizebar-inactive-color
                        Step-gradients:resizebar-inactive-color))
                    (t (list
                        Step-gradients:resizebar-inactive-color2
                        Step-gradients:resizebar-inactive-color2)))))))

    (render-bottom-right-bevel-up (lambda (img state)
        (apply draw-vertical-gradient img
            (if state
                (cond
                    ((eq Step-gradients:resizebar-gradient-type 'vertical) (list 
                        (brighten-color Step-gradients:resizebar-active-color 22000)
                        (brighten-color Step-gradients:resizebar-active-color2 22000)))
                    (t (list
                        (brighten-color Step-gradients:resizebar-active-color2 22000)
                        (brighten-color Step-gradients:resizebar-active-color2 22000))))
                (cond
                    ((eq Step-gradients:resizebar-gradient-type 'vertical) (list 
                        (brighten-color Step-gradients:resizebar-inactive-color 22000)
                        (brighten-color Step-gradients:resizebar-inactive-color2 22000)))
                    (t (list
                        (brighten-color Step-gradients:resizebar-inactive-color2 22000)
                        (brighten-color Step-gradients:resizebar-inactive-color2 22000))))))))

    (render-bottom-right-bevel-down (lambda (img state)
        (apply draw-vertical-gradient img
            (if state
                (cond
                    ((eq Step-gradients:resizebar-gradient-type 'vertical) (list 
                        (darken-color Step-gradients:resizebar-active-color 22000)
                        (darken-color Step-gradients:resizebar-active-color2 22000)))
                    (t (list
                        (darken-color Step-gradients:resizebar-active-color2 22000)
                        (darken-color Step-gradients:resizebar-active-color2 22000))))
                (cond
                    ((eq Step-gradients:resizebar-gradient-type 'vertical) (list 
                        (darken-color Step-gradients:resizebar-inactive-color 22000)
                        (darken-color Step-gradients:resizebar-inactive-color2 22000)))
                    (t (list
                        (darken-color Step-gradients:resizebar-inactive-color2 22000)
                        (darken-color Step-gradients:resizebar-inactive-color2 22000))))))))

    (render-minimize-button (lambda (img state)
        (apply
            (cond
                ((eq Step-gradients:titlebar-gradient-type 'vertical)
                    draw-vertical-gradient)
                (t
                    draw-horizontal-gradient))
            img
            (if state
                (cond
                    ((eq Step-gradients:titlebar-gradient-type 'vertical) (list
                        Step-gradients:titlebar-active-color
                        Step-gradients:titlebar-active-color2))
                    (t (list
                        Step-gradients:titlebar-active-color
                        Step-gradients:titlebar-active-color)))
                (cond
                    ((eq Step-gradients:titlebar-gradient-type 'vertical) (list
                        Step-gradients:titlebar-inactive-color
                        Step-gradients:titlebar-inactive-color2))
                    (t (list
                        Step-gradients:titlebar-inactive-color
                        Step-gradients:titlebar-inactive-color)))))))

    (render-minimize-left (lambda (img state)
        (apply
            (cond
                ((eq Step-gradients:titlebar-gradient-type 'vertical)
                    draw-vertical-gradient)
                (t
                    draw-horizontal-gradient))
            img
            (if state
                (cond
                    ((eq Step-gradients:titlebar-gradient-type 'vertical) (list
                        (brighten-color Step-gradients:titlebar-active-color 22000)
                        (brighten-color Step-gradients:titlebar-active-color2 22000)))
                    (t (list
                        (brighten-color Step-gradients:titlebar-active-color 22000)
                        (brighten-color Step-gradients:titlebar-active-color 22000))))
                (cond
                    ((eq Step-gradients:titlebar-gradient-type 'vertical) (list
                        (brighten-color Step-gradients:titlebar-inactive-color 22000)
                        (brighten-color Step-gradients:titlebar-inactive-color2 22000)))
                    (t (list
                        (brighten-color Step-gradients:titlebar-inactive-color 22000)
                        (brighten-color Step-gradients:titlebar-inactive-color 22000))))))))

    (render-minimize-right (lambda (img state)
        (apply
            (cond
                ((eq Step-gradients:titlebar-gradient-type 'vertical)
                    draw-vertical-gradient)
                (t
                    draw-horizontal-gradient))
            img
            (if state
                (cond
                    ((eq Step-gradients:titlebar-gradient-type 'vertical) (list
                        (darken-color Step-gradients:titlebar-active-color 22000)
                        (darken-color Step-gradients:titlebar-active-color2 22000)))
                    (t (list
                        (darken-color Step-gradients:titlebar-active-color 22000)
                        (darken-color Step-gradients:titlebar-active-color 22000))))
                (cond
                    ((eq Step-gradients:titlebar-gradient-type 'vertical) (list
                        (darken-color Step-gradients:titlebar-inactive-color 22000)
                        (darken-color Step-gradients:titlebar-inactive-color2 22000)))
                    (t (list
                        (darken-color Step-gradients:titlebar-inactive-color 22000)
                        (darken-color Step-gradients:titlebar-inactive-color 22000))))))))

    (render-close-button (lambda (img state)
        (apply
            (cond
                ((eq Step-gradients:titlebar-gradient-type 'vertical)
                    draw-vertical-gradient)
                (t
                    draw-horizontal-gradient))
            img
            (if state
                (cond
                    ((eq Step-gradients:titlebar-gradient-type 'vertical) (list
                        Step-gradients:titlebar-active-color
                        Step-gradients:titlebar-active-color2))
                    ((eq Step-gradients:titlebar-gradient-type 'none) (list
                        Step-gradients:titlebar-active-color
                        Step-gradients:titlebar-active-color))
                    (t (list
                        Step-gradients:titlebar-active-color2
                        Step-gradients:titlebar-active-color2)))
                (cond
                    ((eq Step-gradients:titlebar-gradient-type 'vertical) (list
                        Step-gradients:titlebar-inactive-color
                        Step-gradients:titlebar-inactive-color2))
                    ((eq Step-gradients:titlebar-gradient-type 'none) (list
                        Step-gradients:titlebar-inactive-color
                        Step-gradients:titlebar-inactive-color))
                    (t (list
                        Step-gradients:titlebar-inactive-color2
                        Step-gradients:titlebar-inactive-color2)))))))

    (render-close-left (lambda (img state)
        (apply
            (cond
                ((eq Step-gradients:titlebar-gradient-type 'vertical)
                    draw-vertical-gradient)
                (t
                    draw-horizontal-gradient))
            img
            (if state
                (cond
                    ((eq Step-gradients:titlebar-gradient-type 'vertical) (list
                        (brighten-color Step-gradients:titlebar-active-color 22000)
                        (brighten-color Step-gradients:titlebar-active-color2 22000)))
                    ((eq Step-gradients:titlebar-gradient-type 'none) (list
                        (brighten-color Step-gradients:titlebar-active-color 22000)
                        (brighten-color Step-gradients:titlebar-active-color 22000)))
                    (t (list
                        (brighten-color Step-gradients:titlebar-active-color2 22000)
                        (brighten-color Step-gradients:titlebar-active-color2 22000))))
                (cond
                    ((eq Step-gradients:titlebar-gradient-type 'vertical) (list
                        (brighten-color Step-gradients:titlebar-inactive-color 22000)
                        (brighten-color Step-gradients:titlebar-inactive-color2 22000)))
                    ((eq Step-gradients:titlebar-gradient-type 'none) (list
                        (brighten-color Step-gradients:titlebar-inactive-color 22000)
                        (brighten-color Step-gradients:titlebar-inactive-color 22000)))
                    (t (list
                        (brighten-color Step-gradients:titlebar-inactive-color2 22000)
                        (brighten-color Step-gradients:titlebar-inactive-color2 22000))))))))

    (render-close-right (lambda (img state)
        (apply
            (cond
                ((eq Step-gradients:titlebar-gradient-type 'vertical)
                    draw-vertical-gradient)
                (t
                    draw-horizontal-gradient))
            img
            (if state
                (cond
                    ((eq Step-gradients:titlebar-gradient-type 'vertical) (list
                        (darken-color Step-gradients:titlebar-active-color 22000)
                        (darken-color Step-gradients:titlebar-active-color2 22000)))
                    ((eq Step-gradients:titlebar-gradient-type 'none) (list
                        (darken-color Step-gradients:titlebar-active-color 22000)
                        (darken-color Step-gradients:titlebar-active-color 22000)))
                    (t (list
                        (darken-color Step-gradients:titlebar-active-color2 22000)
                        (darken-color Step-gradients:titlebar-active-color2 22000))))
                (cond
                    ((eq Step-gradients:titlebar-gradient-type 'vertical) (list
                        (darken-color Step-gradients:titlebar-inactive-color 22000)
                        (darken-color Step-gradients:titlebar-inactive-color2 22000)))
                    ((eq Step-gradients:titlebar-gradient-type 'none) (list
                        (darken-color Step-gradients:titlebar-inactive-color 22000)
                        (darken-color Step-gradients:titlebar-inactive-color 22000)))
                    (t (list
                        (darken-color Step-gradients:titlebar-inactive-color2 22000)
                        (darken-color Step-gradients:titlebar-inactive-color2 22000))))))))

    (recolor-image (lambda (img color)
        (when (and (imagep img) (colorp color))
            (let*
                ((rgb (color-rgb color))
                 (red (nth 0 rgb))
                 (green (nth 1 rgb))
                 (blue (nth 2 rgb)))
                (set-image-modifier img 'red (list (/ red 256) 255 (/ red 256)))
                (set-image-modifier img 'green (list (/ green 256) 255 (/ green 256)))
                (set-image-modifier img 'blue (list (/ blue 256) 255 (/ blue 256)))))))

    (minimize-flat (list
        (recolor-image (make-image "minimize-flat-button.png")
            Step-gradients:title-inactive-color)
        (recolor-image (make-image "minimize-flat-button.png")
            Step-gradients:title-active-color)
        nil
        (make-image "minimize-flat-pressed.png")))

    (minimize-bevel (list
        (make-image "minimize-bevel-button.png")
        (make-image "minimize-bevel-button.png")
        nil
        (make-image "minimize-bevel-pressed.png")))

    (minimize (lambda ()
        (if Step-gradients:flat-buttons minimize-flat minimize-bevel)))

    (close-flat (list
        (recolor-image (make-image "close-flat-button.png")
            Step-gradients:title-inactive-color)
        (recolor-image (make-image "close-flat-button.png")
            Step-gradients:title-active-color)
        nil
        (make-image "close-flat-pressed.png")))

    (close-bevel (list
        (make-image "close-bevel-button.png")
        (make-image "close-bevel-button.png")
        nil
        (make-image "close-bevel-pressed.png")))

    (close (lambda ()
        (if Step-gradients:flat-buttons close-flat close-bevel)))

;; --- Frame parts ------------------------
;; Titlebar black border
(black-left `(
    (background . ,border-color)
    (left-edge . -1)
    (width . 1)
    (top-edge . -23)
    (height . 23)))
(black-right `(
    (background . ,border-color)
    (right-edge . -1)
    (width . 1)
    (top-edge . -23)
    (height . 23)))
(black-top `(
    (background . ,border-color)
    (right-edge . -1)
    (left-edge . -1)
    (top-edge . -23)
    (height . 1)))
(black-bottom `(
    (background . ,border-color)
    (right-edge . -1)
    (left-edge . -1)
    (top-edge . -1)
    (height . 1)))
;; Title bevel border
(bevel-left `(
    (renderer . ,render-minimize-left)
    (left-edge . 0)
    (width . 1)
    (top-edge . -22)
    (height . 21)))
(bevel-top `(
    (renderer . ,render-titlebar-bevel-up)
    (right-edge . 0)
    (left-edge . 0)
    (top-edge . -22)
    (height . 1)))
(bevel-right `(
    (renderer . ,render-close-right)
    (right-edge . 0)
    (width . 1)
    (top-edge . -22)
    (height . 21)))
(bevel-bottom `(
    (renderer . ,render-titlebar-bevel-down)
    (right-edge . 0)
    (left-edge . 1)
    (top-edge . -2)
    (height . 1)))

;; Titlebar
(titlebar `(
    (renderer . ,render-titlebar)
    (foreground . ,title-colors)
    (font . ,title-font)
    (text . ,window-name)
    (x-justify . ,text-align)
    (y-justify . center)
    (left-edge . ,(lambda () (if Step-gradients:flat-buttons 23 20)))
    (right-edge . ,(lambda () (if Step-gradients:flat-buttons 23 20)))
    (top-edge . -21)
    (height . 19)
    (cursor . left_ptr)
    (class . title)))

;; Transient window titlebar
(transient-titlebar `(
    (renderer . ,render-titlebar)
    (foreground . ,title-colors)
    (font . ,title-font)
    (text . ,window-name)
    (x-justify . ,text-align)
    (y-justify . center)
    (left-edge . 1)
    (right-edge . ,(lambda () (if Step-gradients:flat-buttons 23 20)))
    (top-edge . -21)
    (height . 19)
    (cursor . left_ptr)
    (class . title)))

;; Resizebar
(resizebar `(
     (renderer . ,render-resizebar)
     (left-edge . 31)
     (right-edge . 31)
     (bottom-edge . -8)
     (height . 6)
;;     (cursor . left_ptr)
     (class . bottom-border)))

;; Resizebar bevel border
(resizebar-bevel-up `(
    (renderer . ,render-resizebar-bevel-up)
    (right-edge . 0)
    (left-edge . 0)
    (bottom-edge . -2)
    (height . 1)))
(resizebar-bevel-down `(
    (renderer . ,render-resizebar-bevel-down)
    (right-edge . 0)
    (left-edge . 0)
    (bottom-edge . -1)
    (height . 1)))

;; Resizebar bottom right corner
(resizebar-bottom-right `(
     (renderer . ,render-bottom-right)
     (right-edge . 0)
     (width . 29)
     (bottom-edge . -8)
     (height . 6)
;;     (cursor . left_ptr)
     (class . bottom-right-corner)))
(resizebar-bottom-right-bevel-up `(
     (renderer . ,render-bottom-right-bevel-up)
     (right-edge . 29)
     (width . 1)
     (bottom-edge . -8)
     (height . 6)))
(resizebar-bottom-right-bevel-down `(
     (renderer . ,render-bottom-right-bevel-down)
     (right-edge . 30)
     (width . 1)
     (bottom-edge . -8)
     (height . 6)))

;; Resizebar bottom left corner
(resizebar-bottom-left `(
    (renderer . ,render-bottom-left)
    (left-edge . 0)
    (width . 29)
    (bottom-edge . -8)
    (height . 6)
;;    (cursor . left_ptr)
    (class . bottom-left-corner)))
(resizebar-bottom-left-bevel-up `(
     (renderer . ,render-bottom-left-bevel-up)
     (left-edge . 30)
     (width . 1)
     (bottom-edge . -8)
     (height . 6)))
(resizebar-bottom-left-bevel-down `(
     (renderer . ,render-bottom-left-bevel-down)
     (left-edge . 29)
     (width . 1)
     (bottom-edge . -8)
     (height . 6)))

;; Minimize button right border
(button-border-11 
    (if Step-gradients:flat-buttons `(
        (renderer . ,render-minimize-right)
        (left-edge . 20)
        (width . ,(lambda () (if Step-gradients:flat-buttons 1 0)))
        (top-edge . -21)
        (height . 20))

        nil))
(button-border-12
    (if Step-gradients:flat-buttons `(
        (background . ,border-color)
        (left-edge . 21)
        (width . ,(lambda () (if Step-gradients:flat-buttons 1 0)))
        (top-edge . -22)
        (height . 21))
    
        nil))
(button-border-13
    (if Step-gradients:flat-buttons `(
        (renderer . ,render-minimize-left)
        (left-edge . 22)
        (width . ,(lambda () (if Step-gradients:flat-buttons 1 0)))
        (top-edge . -22)
        (height . 21))
        
        nil))

;; Minimize button icon
(minimize `(
    (renderer . ,render-minimize-button)
    (foreground . ,minimize)
    (left-edge . 1)
    (width . 19)
    (top-edge . -21)
    (height . 19)
    (cursor . left_ptr)
    (class . iconify-button)))

;; Close button left border
(button-border-21 
    (if Step-gradients:flat-buttons `(
        (renderer . ,render-close-right)
        (right-edge . 22)
        (width . ,(lambda () (if Step-gradients:flat-buttons 1 0)))
        (top-edge . -21)
        (height . 20))
        
        nil))
(button-border-22
    (if Step-gradients:flat-buttons `(
        (background . ,border-color)
        (right-edge . 21)
        (width . ,(lambda () (if Step-gradients:flat-buttons 1 0)))
        (top-edge . -22)
        (height . 21))
        
        nil))
(button-border-23
    (if Step-gradients:flat-buttons `(
        (renderer . ,render-close-left)
        (right-edge . 20)
        (width . ,(lambda () (if Step-gradients:flat-buttons 1 0)))
        (top-edge . -22)
        (height . 21))
        
        nil))

;; Close button icon
(close `(
    (renderer . ,render-close-button)
    (foreground . ,close)
    (right-edge . 1)
    (width . 19)
    (top-edge . -21)
    (height . 19)
    (cursor . left_ptr)
    (class . close-button)))

;; Left border
(left-border `(
    (background . ,border-color)
    (right-edge . -1)
    (width . 1)
    (top-edge . 0)
    (bottom-edge . -1)
;;    (cursor . left_ptr)
    (class . right-border)))

;;Right border
(right-border `(
    (background . ,border-color)
    (left-edge . -1)
    (width . 1)
    (top-edge . 0)
    (bottom-edge . -1)
;;        (cursor . left_ptr)
    (class . left-border)))

;; --- Frame definition -------------------
;; Normal window frame
(frame `(
    ,black-left
    ,black-right
    ,black-top
    ,black-bottom
    ,bevel-left
    ,bevel-right
    ,bevel-top
    ,bevel-bottom
    ,titlebar
    ,button-border-11
    ,button-border-12
    ,button-border-13
    ,minimize
    ,button-border-21
    ,button-border-22
    ,button-border-23
    ,close
    ,left-border
    ,right-border
    ,resizebar-bottom-left-bevel-down
    ,resizebar-bottom-left-bevel-up
    ,resizebar-bottom-left
    ,resizebar-bevel-up
    ,resizebar-bevel-down
    ,resizebar
    ,resizebar-bottom-right-bevel-down
    ,resizebar-bottom-right-bevel-up
    ,resizebar-bottom-right

    ((background . ,border-color)
     (left-edge . -1)
     (width . 1)
     (bottom-edge . -9)
     (height . 9)
;;     (cursor . left_ptr)
     (class . bottom-left-corner))

    ((background . ,border-color)
     (right-edge . -1)
     (left-edge . -1)
     (bottom-edge . -9)
     (height . 1))

    ((background . ,border-color)
     (right-edge . -1)
     (width . 1)
     (bottom-edge . -9)
     (height . 9)
;;     (cursor . left_ptr)
     (class . bottom-right-corner))
)) ;; end frame

;; Normal window shaded frame
(shaped-frame `(
    ,black-left
    ,black-right
    ,black-top
    ,black-bottom
    ,bevel-left
    ,bevel-right
    ,bevel-top
    ,bevel-bottom
    ,titlebar
    ,button-border-11
    ,button-border-12
    ,button-border-13
    ,minimize
    ,button-border-21
    ,button-border-22
    ,button-border-23
    ,close
)) ;; end shaped-frame

;; Dialog window frame
(transient-frame `(
    ,black-left
    ,black-right
    ,black-top
    ,black-bottom
    ,bevel-left
    ,bevel-right
    ,bevel-top
    ,bevel-bottom
    ,transient-titlebar
    ,button-border-21
    ,button-border-22
    ,button-border-23
    ,close
    ,left-border
    ,right-border
;; Bottom border
    ((background . ,border-color)
     (left-edge . 0)
     (right-edge . 0)
     (bottom-edge . -1)
     (height . 1)
;;     (cursor . left_ptr)
     (class . bottom-border))
)) ;; end transient-frame

;; Dialog window shaded frame
(shaped-transient-frame `(
    ,black-left
    ,black-right
    ,black-top
    ,black-bottom
    ,bevel-left
    ,bevel-right
    ,bevel-top
    ,bevel-bottom
    ,transient-titlebar
    ,button-border-21
    ,button-border-22
    ,button-border-23
    ,close
))) ;; end shaped-transient-frame

    (add-frame-style 'Step-gradients
        (lambda (w type)
            (case type
                ((default) frame)
                ((transient) transient-frame)
                ((shaped) shaped-frame)
                ((shaped-transient) shaped-transient-frame))))
)
